import {
  Entity,
  Column,
  PrimaryGeneratedColumn,
  OneToMany,
  ManyToOne,
  OneToOne,
  JoinColumn,
} from 'typeorm';

import { GameEntity } from '../game/game.entity';
import { Game_PersonEntity } from '../game/game.entity';
import { MapDrawingEntity } from '../draw/coordinate.entity';
import { Exclude } from 'class-transformer';
import { ScoreEntity } from '../score/score.entity';

///////////////////////////////////////////////////////////////////
///   FactionEntity & GameGroupEntity reflect database tables.  ///
///////////////////////////////////////////////////////////////////

@Entity('Faction')
export class FactionEntity {
  @PrimaryGeneratedColumn('uuid') factionId: string;
  @Column('text') factionName: string;
  @Column({ type: 'float' }) multiplier: number;
  @Column('text') colour: string;

  // Faction's password won't be included when FactionEntity is transformed
  @Exclude()
  @Column({ type: 'text' })
  factionPassword: string;

  @OneToMany(type => Game_PersonEntity, game_persons => game_persons.faction)
  game_persons: Game_PersonEntity[];
  // When Game where a Faction is in is deleted, the Factions in that game are also deleted
  @ManyToOne(type => GameEntity, game => game.factions, {
    onDelete: 'CASCADE',
  })
  game: GameEntity;
  @OneToMany(type => MapDrawingEntity, mapDrawings => mapDrawings.faction)
  mapDrawings: MapDrawingEntity[];
  @OneToMany(type => ScoreEntity, score => score.scoreId, {
    onDelete: 'NO ACTION',
  })
  scores: ScoreEntity[];

  passwordCheck(pass: string) {
    return pass == this.factionPassword ? true : false;
  }
}

@Entity('GameGroup')
export class GameGroupEntity {
  @PrimaryGeneratedColumn('uuid') id: string;
  @Column('text') name: string;
  @Column('text') class: string;
  // When Groups leader, players or Faction is deleted, the Group(s) go also
  @OneToOne(type => Game_PersonEntity, person => person.leaderGroup, {
    onDelete: 'CASCADE',
  })
  @JoinColumn({ name: 'leader' })
  leader: Game_PersonEntity;
  @OneToMany(type => Game_PersonEntity, person => person.group, {
    onDelete: 'CASCADE',
  })
  players: Game_PersonEntity[];
  @ManyToOne(type => FactionEntity, faction => faction.factionId, {
    onDelete: 'CASCADE',
  })
  faction: FactionEntity;
}
